using System;

using Microsoft.BizTalk.Message.Interop;

namespace Macaw.BizTalk.Pipeline
{
    public abstract class MessageContextBase
    {
        /// <summary>
        /// Constructs a new instance around the <paramref name="messageContext"/>
        /// </summary>
        /// <param name="messageContext">Cannot be null.</param>
        public MessageContextBase(IBaseMessageContext messageContext)
        {
            _msgCtx = messageContext;
        }

        private IBaseMessageContext _msgCtx;
        /// <summary>
        /// Gets the MessageContext.
        /// </summary>
        protected IBaseMessageContext MessageContext
        {
            get { return _msgCtx; }
        }

        protected string Read(string ns, string name, string defaultValue)
        {
            object value = _msgCtx.Read(name, ns);

            if (value != null)
            {
                return Convert.ToString(value);
            }

            return defaultValue;
        }

        protected void WritePromoted(string ns, string name, string value)
        {
            _msgCtx.Promote(name, ns, value);
        }

        protected void WriteDistinguished(string ns, string name, string value)
        {
            _msgCtx.Write(name, ns, value);
        }
    }
}
