using System;
using System.Collections.Generic;
using System.IO;

using Microsoft.BizTalk.Message.Interop;
using Microsoft.BizTalk.Component.Interop;

namespace Macaw.BizTalk.Pipeline
{
    [ComponentCategory(CategoryTypes.CATID_PipelineComponent)]
    [ComponentCategory(CategoryTypes.CATID_Any)]
    [System.Runtime.InteropServices.Guid("0cefabff-48e6-4723-b86b-fafe28fcca85")]
    public class OutboundFileLocation : PipelineComponentWithResourceBase
    {
        public OutboundFileLocation()
            : base("OutboundFileLocation", Properties.Resources.ResourceManager)
        { }

        protected override IBaseMessage ExecuteInternal(IPipelineContext pContext, IBaseMessage pInMsg)
        {
            SystemMessageContext sysCtx = new SystemMessageContext(pInMsg.Context);

            if (sysCtx.OutboundTransportType == "FILE")
            {
                string dir = sysCtx.OutboundTransportLocation;
                int index = dir.IndexOf('%');
                string macro = null;

                if (index >= 0)
                {
                    macro = dir.Substring(index);
                    dir = dir.Substring(0, index);
                }

                foreach(LiteralOrProperty litProp in SubFolders)
                {
                    dir = Path.Combine(dir, litProp.GetValue(pInMsg.Context));
                }

                if (!Directory.Exists(dir))
                {
                    Directory.CreateDirectory(dir);
                }

                if (!String.IsNullOrEmpty(macro))
                {
                    dir = Path.Combine(dir, macro);
                }

                // assign new directory
                sysCtx.OutboundTransportLocation = dir;
            }

            return pInMsg;
        }

        private List<LiteralOrProperty> _properties = 
            new List<LiteralOrProperty>();
        public List<LiteralOrProperty> SubFolders
        {
            get { return _properties; }
        }

        #region IPersistPropertyBag implementation

        public override void Load(IPropertyBag propertyBag, int errorLog)
        {
            base.Load(propertyBag, errorLog);

            int count = base.ReadProperty<int>(
                propertyBag, OutboundFileLocationBagNames.Count, 0);

            for (int index = 0; index < count; index++)
            {
                LiteralOrProperty ctxProp = new LiteralOrProperty();

                ctxProp.Key = ReadProperty<string>(propertyBag,
                    OutboundFileLocationBagNames.FormatKey(index), String.Empty);
                ctxProp.Namespace = ReadProperty<string>(propertyBag,
                    OutboundFileLocationBagNames.FormatNamespace(index), String.Empty);
                ctxProp.IsLiteral = ReadProperty<bool>(propertyBag,
                    OutboundFileLocationBagNames.FormatIsLiteral(index), true);

                SubFolders.Add(ctxProp);
            }
        }

        public override void Save(IPropertyBag propertyBag, bool clearDirty, bool saveAllProperties)
        {
            base.Save(propertyBag, clearDirty, saveAllProperties);

            int count = SubFolders.Count;
            WriteProperty(propertyBag, OutboundFileLocationBagNames.Count, count);

            for (int index = 0; index < count; index++)
            {
                LiteralOrProperty ctxProp = SubFolders[index];

                WriteProperty(propertyBag,
                    OutboundFileLocationBagNames.FormatKey(index), ctxProp.Key);
                WriteProperty(propertyBag,
                    OutboundFileLocationBagNames.FormatNamespace(index), ctxProp.Namespace);
                WriteProperty(propertyBag,
                    OutboundFileLocationBagNames.FormatIsLiteral(index), ctxProp.IsLiteral);
            }
        }

        #endregion
    }
}
